/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef __NANO1XX_ADC_H__
#define __NANO1XX_ADC_H__


/*---------------------------------------------------------------------------------------------------------*/
/* Include related headers                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
#include "nano1xx.h"
//#include "System/SysInfra.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
/* version definition with SysInfra */
#define	DRVADC_MAJOR_NUM 1
#define	DRVADC_MINOR_NUM 00
#define	DRVADC_BUILD_NUM 000
#define DRVADC_VERSION_NUM    _SYSINFRA_VERSION(DRVADC_MAJOR_NUM, DRVADC_MINOR_NUM, DRVADC_BUILD_NUM)

/* error code definition */
#define E_DRVADC_ARGUMENT           _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVADC, 1)

/** There are 11 ADC channels, 0~10 */
#define ADC_CH_NUM          11

typedef enum {
	ADC_SINGLE_OP = 0, 
	ADC_SINGLE_CYCLE_OP = 2, 
	ADC_CONTINUOUS_OP = 3
} E_ADC_OPERATION_MODE;

typedef enum {
	ADC_CLK_HXT = 0,  
	ADC_CLK_LXT = 1, 
	ADC_CLK_PLL = 2, 
	ADC_CLK_HIRC = 3
} E_ADC_CLK_SRC;

typedef enum {
	ADC_LOW_LEVEL=0, 
	ADC_HIGH_LEVEL=1, 
	ADC_FALLING_EDGE=2, 
	ADC_RISING_EDGE=3
} E_ADC_EXT_TRI_COND;

/** Indicates ADC channel 8 input */
typedef enum {
	ADC_CH10_VTEMP,   ///< Internal temperature sensor as channel 8 input
	ADC_CH10_VBG,     ///< Bandgap voltage as channel 8 input
	ADC_CH10_AVDD,    ///< Analog VDD as channel 8 input
	ADC_CH10_AVSS     ///< Analog VSS as channel 8 input
} E_ADC_CH10_SRC;

/**
 *  Indicates reference voltage source
 */
typedef enum {
	ADC_REF_POWER = 0,   ///< Input power as reference voltage
	ADC_REF_VBG = 1,     ///< Bandgap as reference voltage
	ADC_REF_EXT = 2,     ///< External power as reference voltage
	ADC_REF_CH7 = 3      ///< Channel 7 input as reference voltage
}E_ADC_REF;

typedef enum {
	ADC_LESS_THAN = 0, 
	ADC_GREATER_OR_EQUAL = 4
} E_ADC_CMP_CONDITION;

typedef enum {
	ADC_UNSIGNED_OUTPUT = 0, 
	ADC_TWOS_COMPLEMENT = 1
} E_ADC_DIFF_MODE_OUTPUT_FORMAT;

/** Indicates timer to triger time out event convresion */
typedef enum {
	ADC_TIMER0_TRIGGER = 0,   ///< Timer 0 as time out source 
	ADC_TIMER1_TRIGGER = 1,   ///< Timer 1 as time out source
	ADC_TIMER2_TRIGGER = 2,   ///< Timer 2 as time out source
	ADC_TIMER3_TRIGGER = 3,   ///< Timer 3 as time out source 
}E_ADC_TIMER_TRIGGER_SRC;

/** Stores timer trigger relate settings  */
typedef struct {
	E_ADC_TIMER_TRIGGER_SRC src;      ///< Indicates timer trigger source
	uint8_t conversion_cnt;           ///< Conversion count for each iteration before go back to sleep
}S_ADC_TIMER_TRIGGER_CFG;

typedef void (DRVADC_ADC_CALLBACK)(uint32_t u32UserData);
typedef void (DRVADC_ADCMP0_CALLBACK)(uint32_t u32UserData);
typedef void (DRVADC_ADCMP1_CALLBACK)(uint32_t u32UserData);

#define _DRVADC_CONV() do{ADC->CR_BITS.ADST=1k}while(0)
#define _DRVADC_GET_ADC_INT_FLAG() do{ADC->SR_BITS.ADFk}while(0)
#define _DRVADC_GET_CMP0_INT_FLAG() do{ADC->SR_BITS.CMPF0k}while(0)
#define _DRVADC_GET_CMP1_INT_FLAG() do{ADC->SR_BITS.CMPF1k}while(0)
#define _DRVADC_CLEAR_ADC_INT_FLAG() do{ADC->SR = 1;}while(0)
#define _DRVADC_CLEAR_CMP0_INT_FLAG() do{ADC->SR = 2;}while(0)
#define _DRVADC_CLEAR_CMP1_INT_FLAG() do{ADC->SR = 4;}while(0)


/*---------------------------------------------------------------------------------------------------------*/
/* Define Function Prototypes                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
void DrvADC_Open(E_ADC_OPERATION_MODE OpMode, 
				uint16_t u16ChannelSelBitwise, 
				E_ADC_CLK_SRC ClockSrc, 
				uint8_t u8AdcDivisor,
				E_ADC_REF ref);
void DrvADC_Close(void);
void DrvADC_SetADCChannel(uint16_t u16ChannelSelBitwise);
void DrvADC_ConfigADCChannel10(E_ADC_CH10_SRC Ch10Src);
void DrvADC_SetADCOperationMode(E_ADC_OPERATION_MODE OpMode);
void DrvADC_SetADCClkSrc(E_ADC_CLK_SRC ClockSrc);
void DrvADC_SetADCDivisor(uint8_t u8AdcDivisor);
void DrvADC_EnableADCInt(DRVADC_ADC_CALLBACK callback, uint32_t u32UserData);
void DrvADC_DisableADCInt(void);
void DrvADC_EnableADCCmp0Int(DRVADC_ADCMP0_CALLBACK callback, uint32_t u32UserData);
void DrvADC_DisableADCCmp0Int(void);
void DrvADC_EnableADCCmp1Int(DRVADC_ADCMP1_CALLBACK callback, uint32_t u32UserData);
void DrvADC_DisableADCCmp1Int(void);
uint32_t DrvADC_GetConversionRate(void);
void DrvADC_EnableExtTrigger(E_ADC_EXT_TRI_COND TriggerCondition);
void DrvADC_DisableExtTrigger(void);
void DrvADC_StartConvert(void);
void DrvADC_StopConvert(void);
uint32_t DrvADC_IsConversionDone(void);
int32_t DrvADC_GetConversionData(uint8_t u8ChannelNum);
void DrvADC_EnablePDMA(void);
void DrvADC_DisablePDMA(void);
uint32_t DrvADC_IsDataValid(uint8_t u8ChannelNum);
uint32_t DrvADC_IsDataOverrun(uint8_t u8ChannelNum);
int32_t DrvADC_EnableADCCmp0(uint8_t u8CmpChannelNum, 
                             E_ADC_CMP_CONDITION CmpCondition, 
							 uint16_t u16CmpData, 
							 uint8_t CmpMatchCount);
int32_t DrvADC_EnableADCCmp1(uint8_t u8CmpChannelNum, 
                             E_ADC_CMP_CONDITION CmpCondition, 
							 uint16_t u16CmpData, 
							 uint8_t CmpMatchCount);
void DrvADC_DisableADCCmp0(void);
void DrvADC_DisableADCCmp1(void);
uint32_t DrvADC_GetVersion (void);
void DrvADC_Reset(void);
int32_t DrvADC_EnableTimerTrigger(S_ADC_TIMER_TRIGGER_CFG *cfg);
void DrvADC_DisableTimerTrigger(void);
#endif //__NANO1XX_ADC_H__
