/**
  * COPYRIGHT 2010 Nuvoton Technology Corp.
  ******************************************************************************
  * @file    DrvDAC.h
  * @author  NuMicro MCU Software Team
  * @version V1.0.0
  * @date    08/11/2010
  * @brief   NANO1XX DAC driver header file
  ******************************************************************************
  */ 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "nano1xx.h"

#ifndef _NANO1XX_DAC_H_
#define _NANO1XX_DAC_H_


#ifdef  __cplusplus
extern "C"
{
#endif

/* version definition with SysInfra */
#define	DRVDAC_MAJOR_NUM 1
#define	DRVDAC_MINOR_NUM 00
#define	DRVDAC_BUILD_NUM 000
#define DRVDAC_VERSION_NUM    _SYSINFRA_VERSION(DRVDAC_MAJOR_NUM, DRVDAC_MINOR_NUM, DRVDAC_BUILD_NUM)

#define E_DRVDAC_ALREADY          _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVDAC, 1)  /* Already/open/close        */
#define E_DRVDAC_PARAM            _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVDAC, 1)  /* Invalid parameter         */
#define E_DRVDAC_PERM             _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVDAC, 1)  /* No permission             */
#define E_DRVDAC_AGAIN            _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVDAC, 1)  /* Busy, try again later     */


/** Max valid channel number. NANO100 supports 2 DAC channels, 0~1 */
#define DAC_MAX_CH      1

/**
 * Indicates the time out source to wake up DAC from power off state
 */
typedef enum {
    DAC_WAKE_SRC_TIMER0 = 0,    ///< Wakeup with timer0 time out event
    DAC_WAKE_SRC_TIMER1,        ///< Wakeup with timer1 time out event
	DAC_WAKE_SRC_TIMER2,        ///< Wakeup with timer2 time out event
	DAC_WAKE_SRC_TIMER3         ///< Wakeup with timer3 time out event
}E_DAC_WAKE_SRC;

/**
 * Indicates the the load trigger for the DAC latch.
 */
typedef enum {
	DAC_LOAD_DAT = 0,      ///< Load after DAC_DATx writtren
	DAC_LOAD_PDMA,         ///< Load after PDMA ACKed 
	DAC_LOAD_TIMER0,       ///< Load after timer0 time out
	DAC_LOAD_TIMER1,       ///< Load after timer1 time out
	DAC_LOAD_TIMER2,       ///< Load after timer2 time out
	DAC_LOAD_TIMER3,       ///< Load after timer3 time out
	DAC_LOAD_WAKEUP        ///< Load after wakeup by timer source idicates in E_DAC_WAKE_SRC.
}E_DAC_LOAD_SEL;

/** Indicates DAC refreence voltage */
typedef enum {
	DAC_REF_INT = 0,          ///< internal reference voltage
	DAC_REF_EXT = 1,          ///< external reference voltage
    DAC_REF_DAC_VDD  = 2      ///< DAC VDD as reference voltage
}E_DAC_REF_VOL;


/** Indicates DAC channel's status */
typedef enum {
	DAC_IDLE,          ///< DAC channel is idle, can output new data
	DAC_BUSY,          ///< DAC channel is converting data
	DAC_INIT           ///< DAC is initializing
}E_DAC_STATE;

/**
 * Stores DAC configurations for single channel
 */
typedef struct {
	E_DAC_WAKE_SRC eWakeSrc;    ///< Wake up timer source
	E_DAC_LOAD_SEL eLoadSel;    ///< Load trigger selection
	uint8_t u8TransCnt;         ///< Transfer count on each PDMA transfer
}S_DAC_CFG;

/** DAC interrupt callback function prototype */
typedef void (*PFN_DAC_CALLBACK)(uint32_t u32Param);

extern int32_t DrvDAC_Open(uint8_t u8Ch, S_DAC_CFG *psCfg, uint8_t u8Delay);
extern int32_t DrvDAC_Close(uint8_t u8Ch);
extern void DrvDAC_Reset(void);
extern int32_t DrvDAC_WriteData(uint8_t u8Ch, uint16_t u16Data);
extern int32_t DrvDAC_SetGroup(uint8_t u8Enable);
extern int32_t DrvDAC_SetDelayTime(uint8_t u8Delay); // this setting effect both channel
extern void DrvDAC_SetRefVol(E_DAC_REF_VOL eRefVol);
extern int32_t DrvDAC_SetWakeSrc(uint8_t u8Ch, E_DAC_WAKE_SRC eWakeSrc);
extern int32_t DrvDAC_SetLoadSel(uint8_t u8Ch, E_DAC_LOAD_SEL eLoadSel);
extern int32_t DrvDAC_RegisterCallback(uint8_t u8Ch, PFN_DAC_CALLBACK pfnFp, uint32_t u32Param);
extern int32_t DrvDAC_EnableInterrupt(uint8_t u8Ch);
extern int32_t DrvDAC_DisableInterrupt(uint8_t u8Ch);
extern int32_t DrvDAC_GetState(uint8_t u8Ch, E_DAC_STATE *peState);
#ifdef  __cplusplus
}
#endif

#endif //#ifndef _NANO1XX_DAC_H_

/*** (C) COPYRIGHT 2010 Nuvoton Technology Corp. ***/

