/**
  * COPYRIGHT 2011 Nuvoton Technology Corp.
  ******************************************************************************
  * @file    DrvLCD.h
  * @author  NuMicro MCU Software Team
  * @version  V1.06
  * @date     14. March 2011
  * @brief   This file provides all the LCD firmware functions.
  ******************************************************************************
  */
#ifndef __NANO1XX_LCD_H
#define __NANO1XX_LCD_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include <stdbool.h>
#include "NANO1xx.h"



/** @addtogroup NANO1XX_StdPeriph_Driver
  * @{
  */

/** @addtogroup LCD
  * @{
  */


/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVLCD_MAJOR_NUM	 1
#define DRVLCD_MINOR_NUM	 3
#define DRVLCD_BUILD_NUM	 1

/* Define module version number */
#define DRVLCD_VERSION_NUM    _SYSINFRA_VERSION(DRVLCD_MAJOR_NUM, DRVLCD_MINOR_NUM, DRVLCD_BUILD_NUM)



/** @defgroup LCD_Exported_Types
  * @{
  */  

/** @defgroup E_LCDSYMBOL
  * @{
  */
#ifdef __LCDDISPLAY_GFN616A_H  
typedef enum
{
	LCD_SYMBOL_S1,						/* 0 */
	LCD_SYMBOL_S2,						/* 1 */
	LCD_SYMBOL_S3,						/* 2 */
	LCD_SYMBOL_S4,						/* 3 */
	LCD_SYMBOL_S5,						/* 4 */
	LCD_SYMBOL_S_EXCLAMATION,		/* 5 */
	LCD_SYMBOL_EXCLAMATION,		/* 6 */
	LCD_SYMBOL_PT,						/* 7 */
	LCD_SYMBOL_CT,						/* 8 */
	LCD_SYMBOL_LEFTARROW,			/* 9 */
	LCD_SYMBOL_RIGHTARROW,			/* 10 */
	LCD_SYMBOL_COM,					/* 11 */
	LCD_SYMBOL_BAL,					/* 12 */
	LCD_SYMBOL_PAR,					/* 13 */
	LCD_SYMBOL_T1,						/* 14 */
	LCD_SYMBOL_T2,						/* 15 */
	LCD_SYMBOL_T3,						/* 16 */
	LCD_SYMBOL_T4,						/* 17 */
	LCD_SYMBOL_L1,						/* 18 */
	LCD_SYMBOL_L2,						/* 19 */
	LCD_SYMBOL_L3,						/* 20 */
	LCD_SYMBOL_11,						/* 21 */
	LCD_SYMBOL_21,						/* 22 */
	LCD_SYMBOL_22,						/* 23 */
	LCD_SYMBOL_32,						/* 24 */
	LCD_SYMBOL_33,						/* 25 */
	LCD_SYMBOL_13,						/* 26 */
	LCD_SYMBOL_PULSEONE,			/* 27 */
	LCD_SYMBOL_PULSETWO,			/* 28 */
	LCD_SYMBOL_UPDOWN,				/* 29 */
	LCD_SYMBOL_DASH,					/* 30 */
	LCD_SYMBOL_MRIGHTSIDE,			/* 31 */
	LCD_SYMBOL_MLEFTSIDE,			/* 32 */
	LCD_SYMBOL_COL,					/* 33 */
	LCD_SYMBOL_M1,						/* 34 */
	LCD_SYMBOL_K1,						/* 35 */
	LCD_SYMBOL_V,						/* 36 */
	LCD_SYMBOL_A,						/* 37 */
	LCD_SYMBOL_H2,						/* 38 */
	LCD_SYMBOL_N,						/* 39 */
	LCD_SYMBOL_M2,						/* 40 */
	LCD_SYMBOL_K2,						/* 41 */
	LCD_SYMBOL_W,						/* 42 */
	LCD_SYMBOL_H3,						/* 43 */
	LCD_SYMBOL_M3,						/* 44 */
	LCD_SYMBOL_K3,						/* 45 */
	LCD_SYMBOL_VAR,					/* 46 */
	LCD_SYMBOL_H1,						/* 47 */
	LCD_SYMBOL_TH,						/* 48 */
	LCD_SYMBOL_D,						/* 49 */
	LCD_SYMBOL_PF,						/* 50 */
	LCD_SYMBOL_HZ,						/* 51 */
	LCD_SYMBOL_DMD,					/* 52 */
	LCD_SYMBOL_MAX,					/* 53 */
	LCD_SYMBOL_MIN,					/* 54 */
	LCD_SYMBOL_ERR,					/* 55 */
} E_LCDSYMBOL;
#endif
#ifdef __LCDDISPLAY_PD_878_H
typedef enum
{
	LCD_SYMBOL_XXX,						/* 0 */
} E_LCDSYMBOL;
#endif
#ifdef __LCDDISPLAY_BTL001_H
typedef enum
{
	LCD_SYMBOL_PLUS,					/* 0 */
	LCD_SYMBOL_MINUS,					/* 1 */
	LCD_SYMBOL_1DP,					/* 2 */
	LCD_SYMBOL_2DP,					/* 3 */
	LCD_SYMBOL_3DP,					/* 4 */
	LCD_SYMBOL_4DP,					/* 5 */
	LCD_SYMBOL_5DP,					/* 6 */
	LCD_SYMBOL_6DP,					/* 7 */
	LCD_SYMBOL_8DP,					/* 8 */
	LCD_SYMBOL_10DP,					/* 9 */
	LCD_SYMBOL_11DP,					/* 10 */
	LCD_SYMBOL_2COL,					/* 11 */
	LCD_SYMBOL_3COL,					/* 12 */
	LCD_SYMBOL_4COL,					/* 13 */
	LCD_SYMBOL_5COL,					/* 14 */
	LCD_SYMBOL_6COL,					/* 15 */
	LCD_SYMBOL_10COL,					/* 16 */
	LCD_SYMBOL_PL,						/* 17 */
	LCD_SYMBOL_P0,						/* 18 */
	LCD_SYMBOL_P1,						/* 19 */
	LCD_SYMBOL_P2,						/* 20 */
	LCD_SYMBOL_P3,						/* 21 */
	LCD_SYMBOL_P4,						/* 22 */
	LCD_SYMBOL_P5,						/* 23 */
	LCD_SYMBOL_P6,						/* 24 */
	LCD_SYMBOL_P7,						/* 25 */
	LCD_SYMBOL_P8,						/* 26 */
	LCD_SYMBOL_P9,						/* 27 */
	LCD_SYMBOL_PR,						/* 28 */
	LCD_SYMBOL_BRBL,					/* 29 */
	LCD_SYMBOL_B0,						/* 30 */
	LCD_SYMBOL_B1,						/* 31 */
	LCD_SYMBOL_B2,						/* 32 */	
	LCD_SYMBOL_SB,						/* 33 */	
} E_LCDSYMBOL;
#endif
#ifdef __LCDDISPLAY_BTL001_LB_H
typedef enum
{
	LCD_SYMBOL_3DP,					/* 0 */
	LCD_SYMBOL_4DP,					/* 1 */
	LCD_SYMBOL_5DP,					/* 2 */
	LCD_SYMBOL_6DP,					/* 3 */
	LCD_SYMBOL_11DP,					/* 4 */
	LCD_SYMBOL_4COL,					/* 5 */
	LCD_SYMBOL_5COL,					/* 6 */
	LCD_SYMBOL_6COL,					/* 7 */
	LCD_SYMBOL_10COL,					/* 8 */
	LCD_SYMBOL_PL,						/* 9 */
	LCD_SYMBOL_P0,						/* 10 */
	LCD_SYMBOL_P1,						/* 11 */
	LCD_SYMBOL_P2,						/* 12 */
	LCD_SYMBOL_P3,						/* 13 */
	LCD_SYMBOL_P4,						/* 14 */
	LCD_SYMBOL_P5,						/* 15 */
	LCD_SYMBOL_P6,						/* 16 */
	LCD_SYMBOL_P7,						/* 17 */
	LCD_SYMBOL_P8,						/* 18 */
	LCD_SYMBOL_P9,						/* 19 */
	LCD_SYMBOL_PR,						/* 20 */
	LCD_SYMBOL_BRBL,					/* 21 */
	LCD_SYMBOL_B0,						/* 22 */
	LCD_SYMBOL_B1,						/* 23 */
	LCD_SYMBOL_B2,						/* 24 */	
	LCD_SYMBOL_SB,						/* 25 */	
} E_LCDSYMBOL;
#endif

/**
  * @}
  */ 
  

/** @defgroup E_LCD_FREQDIV
  * @{
  */
typedef enum
{
	LCD_FREQ_DIV32 = 0,			/*!< Clock source (32 or 10KHz) divide by 32 */
	LCD_FREQ_DIV64 = 1,			/*!< Clock source (32 or 10KHz) divide by 64 */
	LCD_FREQ_DIV96 = 2,			/*!< Clock source (32 or 10KHz) divide by 96 */
	LCD_FREQ_DIV128 = 3,			/*!< Clock source (32 or 10KHz) divide by 128 */
	LCD_FREQ_DIV192 = 4,			/*!< Clock source (32 or 10KHz) divide by 192 */
	LCD_FREQ_DIV256 = 5,			/*!< Clock source (32 or 10KHz) divide by 256 */
	LCD_FREQ_DIV384 = 6,			/*!< Clock source (32 or 10KHz) divide by 384 */
	LCD_FREQ_DIV512 = 7			/*!< Clock source (32 or 10KHz) divide by 512 */
} E_LCD_FREQDIV;
/**
  * @}
  */ 

/** @defgroup E_LCD_MUX
  * @{
  */
typedef enum
{
	LCD_MUX_STATIC 		= 0,	/*!< Static multiplexing */
	LCD_MUX_DUPLEX		= 1,	/*!< Duplex multiplexing */
	LCD_MUX_TRIPLE		= 2,	/*!< Triplex multiplexing */
	LCD_MUX_QUADRUPLEX 	= 3		/*!< Quadruplex multiplexing */
} E_LCD_MUX;
/**
  * @}
  */


/** @defgroup E_LCD_BIAS
  * @{
  */
typedef enum
{
	LCD_BIAS_STATIC 	= 0,	/*!< Static bias */
	LCD_BIAS_HALF		= 1,	/*!< 1/2 bias */
	LCD_BIAS_THIRD	= 2		/*!< 1/3 bias */
} E_LCD_BIAS;
/**
  * @}
  */


/** @defgroup E_LCD_CPUMP_FREQDIV
  * @{
  */
typedef enum
{
	LCD_CPUMP_DIV1 = 0,		/*!< Clock source (32 or 10KHz) divide by 1 and is used by analog component */
	LCD_CPUMP_DIV2 = 1,		/*!< Clock source (32 or 10KHz) divide by 2 */
	LCD_CPUMP_DIV4 = 2,		/*!< Clock source (32 or 10KHz) divide by 4 */
	LCD_CPUMP_DIV8 = 3,		/*!< Clock source (32 or 10KHz) divide by 8 */
	LCD_CPUMP_DIV16 = 4,		/*!< Clock source (32 or 10KHz) divide by 16 */
	LCD_CPUMP_DIV32 = 5,		/*!< Clock source (32 or 10KHz) divide by 32 */
	LCD_CPUMP_DIV64 = 6,		/*!< Clock source (32 or 10KHz) divide by 64 */
	LCD_CPUMP_DIV128 = 7		/*!< Clock source (32 or 10KHz) divide by 128 */
} E_LCD_CPUMP_FREQDIV;
/**
  * @}
  */ 
  

/** @defgroup E_LCD_CPUMP_Vol
  * @{
  */
typedef enum
{
	LCD_CPVOl_2_6V = 0,		/*!< Set charge pump voltage to 2.6 V */
	LCD_CPVOl_2_7V = 1,		/*!< Set charge pump voltage to 2.7 V */
	LCD_CPVOl_2_8V = 2,		/*!< Set charge pump voltage to 2.8 V */
	LCD_CPVOl_2_9V = 3,		/*!< Set charge pump voltage to 2.9 V */
	LCD_CPVOl_3V 	 = 4,		/*!< Set charge pump voltage to 3 V */
	LCD_CPVOl_3_1V = 5,		/*!< Set charge pump voltage to 3.1 V */
	LCD_CPVOl_3_2V = 6,		/*!< Set charge pump voltage to 3.2 V */
	LCD_CPVOl_3_3V = 7		/*!< Set charge pump voltage to 3.3 V */
} E_LCD_CPUMP_VOL;
/**
  * @}
  */ 
  

/** @defgroup S_LCD_INIT
  * @{
  */
typedef struct
{
	bool	cpump_enable;						/*!< Specifies if enable charge pump or not */

	bool	internal_bias;						/*!< Specifies if enable internal bias reference ladder or not */	
	
	E_LCD_MUX     	mux;						/*!< Specifies Mux configuration */

	E_LCD_BIAS    	bias;						/*!< Specifies Bias configuration */

	E_LCD_FREQDIV freqdiv;					/*!< Specifies LCD Frequency Divider Selection */	

	E_LCD_CPUMP_FREQDIV		cpump_freqdiv;		/*!< Specifies frequency divider of charge pump */
	E_LCD_CPUMP_VOL			cpump_voltage;		/*!< Specifies voltage of charge pump */
} S_LCD_INIT;
/**
  * @}
  */



/** @defgroup E_LCD_FC_PRESCALE
  * @{
  */
typedef enum
{
	LCD_FCPRESC_DIV1 = 0,		/*!< Set pre-scale divider value to 1 */
	LCD_FCPRESC_DIV2 = 1,		/*!< Set pre-scale divider value to 2 */
	LCD_FCPRESC_DIV4 = 2,		/*!< Set pre-scale divider value to 4 */
	LCD_FCPRESC_DIV8 = 3			/*!< Set pre-scale divider value to 8 */
} E_LCD_FC_PRESCALE;
/**
  * @}
  */


/** @defgroup E_LCD_INTSOURCE
  * @{
  */
typedef enum
{
	LCD_FRAMECOUNT_INT    	= 0x01,		/*!< Indicate frame count interrupt */
	LCD_POWERDOWN_INT     	= 0x02,		/*!< Indicate power down interrupt */
	LCD_ALL_INT					= 0x03		/*!< Indicate frame count & power down interrupt */
} E_LCD_INTSOURCE;
/**
  * @}
  */


/** @defgroup LCD_CallBack_Function_Pointer
  * @{
  */
typedef void (PFN_LCD_CALLBACK)(void);
/**
  * @}
  */


/** @defgroup LCD_Return_Error_Code
  * @{
  */
#define ERR_LCD_CAL_BLINK_FAIL		0xFFFF0000				/*!< Specifies that overflow to calculate the blinking frequency */
/**
  * @}
  */

/** @defgroup LCD_Pin_Assignment
  * @{
  */
#define LCD_LQFP100_PIN				/*!< If ROMMAP[0] Bit[10] is clear and Bit[9] is set */
//#define LCD_LQFP64_PIN				/*!< If ROMMAP[0] Bit[9] is clear and Bit[10] is set */
/**
  * @}
  */ 
  

/**
  * @}
  */


/** @defgroup LCD_Exported_Functions
  * @{
  */
void DrvLCD_Init(S_LCD_INIT *lcdinit);
void DrvLCD_ChargePumpInit(S_LCD_INIT *lcdinit);
void DrvLCD_IRQHandler(void);


void DrvLCD_AllOff(void);
void DrvLCD_AllOn(void);

void DrvLCD_FrameCountEnable( E_LCD_FC_PRESCALE prediv, uint32_t fcv );
void DrvLCD_FrameCountDisable(void);

void DrvLCD_EnableSegment(int com, int bitvalue);
void DrvLCD_DisableSegment(int com, int bitvalue);


void DrvLCD_Number(long long value);
void DrvLCD_NumberOff(void);
void DrvLCD_AlphaNumberOff(void);
void DrvLCD_Symbol(E_LCDSYMBOL s, int on);
void DrvLCD_Write(char *string);

int32_t DrvLCD_BlinkFrequency(double mdelay);
void DrvLCD_EnableBlink(void);
void DrvLCD_DisableBlink(void);

void DrvLCD_CLKSrcSel(int src);
void DrvLCD_ContrastSet(E_LCD_CPUMP_VOL level);

void DrvLCD_Enable(void);
void DrvLCD_Disable(void);

void DrvLCD_PowerDownDisplay(int32_t keepdpy);

void DrvLCD_EnableCallbackInt(E_LCD_INTSOURCE IntSrc, PFN_LCD_CALLBACK pfncallback);
void DrvLCD_DisableCallbackInt(E_LCD_INTSOURCE IntSrc);

void DrvLCD_SegmentPinEnable(uint32_t pinno, bool enable);
void DrvLCD_UpdateLCDFrameFreq(void);
void DrvLCD_UpdateChargePumpFreq(void);
int32_t	DrvLCD_GetVersion(void);

#ifdef __cplusplus
}
#endif

#endif	/* __NANO1XX_LCD_H */

/**
  * @}
  */ 

/**
  * @}
  */ 

/**
  * @}
  */ 


/*** (C) COPYRIGHT 2011 Nuvoton Technology Corp. ***/


