/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef DRVPDMA_H
#define DRVPDMA_H

/*---------------------------------------------------------------------------------------------------------*/
/* Includes of local headers                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
#include "NANO1xx.h"


#ifdef  __cplusplus
extern "C"
{
#endif


#define CHANNEL_OFFSET 0x100
/*---------------------------------------------------------------------------------------------------------*/
/* Version number                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPDMA_MAJOR_NUM 1
#define DRVPDMA_MINOR_NUM 01
#define DRVPDMA_BUILD_NUM 001

							    
#define DRVPDMA_VERSION_NUM     _SYSINFRA_VERSION(DRVPDMA_MAJOR_NUM, DRVPDMA_MINOR_NUM, DRVPDMA_BUILD_NUM)

/*---------------------------------------------------------------------------------------------------------*/
/* Error Code define                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
// E_DRVPDMA_FALSE_INPUT		    Non-support specified parameter
// E_DRVPDMA_ERR_PORT_INVALID		Invaild port parameter
#define E_DRVPDMA_FALSE_INPUT   	_SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVPDMA, 1)
#define E_DRVPDMA_ERR_PORT_INVALID	_SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVPDMA, 2)

/*---------------------------------------------------------------------------------------------------------*/
/* For interrupt CallBack Function                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
typedef void (*PFN_DRVPDMA_CALLBACK)(uint32_t);

/*---------------------------------------------------------------------------------------------------------*/
/* Enumerate Type                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/

typedef enum										/* PDMA Operation Channel Seletion */
{
	eDRVPDMA_CHANNEL_0	=	0,
	eDRVPDMA_CHANNEL_1 	=	1,
	eDRVPDMA_CHANNEL_2	=	2,
	eDRVPDMA_CHANNEL_3	=	3,
	eDRVPDMA_CHANNEL_4	=	4,
	eDRVPDMA_CHANNEL_5 	=	5,
	eDRVPDMA_CHANNEL_6	=	6,
	eDRVPDMA_CHANNEL_7	=	7,
	eDRVPDMA_CHANNEL_8	=   8
	
}E_DRVPDMA_CHANNEL_INDEX;


typedef enum										/* PDMA Interrupt Flag */
{
	eDRVPDMA_TABORT_FLAG	= 0x01,
	eDRVPDMA_BLKD_FLAG		= 0x02,
	eDRVPDMA_WRAP_TC_FLAG	= 0x04,		/* wrap around transfer complete */
	eDRVPDMA_WRAP_HT_FLAG	= 0x10,		/* wrap around transfer equal 1/2 */
	eDRVPDMA_TIMEOUT_FLAG	= 0x40
}E_DRVPDMA_INT_FLAG;

typedef enum
{													/* PDMA Source/ Destination Transfer Selection structure */
	eDRVPDMA_DIRECTION_INCREMENTED 	=	0,
	eDRVPDMA_DIRECTION_FIXED      	=	2,
	eDRVPDMA_DIRECTION_WRAPAROUND  	=	3
}E_DRVPDMA_DIRECTION_SELECT;

typedef enum										/* PDMA Transfer width selection */
{
	eDRVPDMA_WIDTH_32BITS	= 0,    
	eDRVPDMA_WIDTH_8BITS 	= 1,
	eDRVPDMA_WIDTH_16BITS	= 2
}E_DRVPDMA_TRANSFER_WIDTH;

typedef enum										/* PDMA Interrupt */
{
	eDRVPDMA_TABORT 	= 0x01,
	eDRVPDMA_BLKD		= 0x02,
	eDRVPDMA_WRAP_TC	= 0x04,		/* wrap around transfer complete */
	eDRVPDMA_WRAP_HT	= 0x10,		/* wrap around transfer equal 1/2 */
	eDRVPDMA_TIMEOUT	= 0x40
}E_DRVPDMA_INT_ENABLE;


typedef enum										/* PDMA APB Device selection */
{
	eDRVPDMA_SPI0_TX	= 0x00,
	eDRVPDMA_SPI1_TX	= 0x01,
	eDRVPDMA_UART0_TX	= 0x02,
	eDRVPDMA_UART1_TX	= 0x03,
	eDRVPDMA_USB_TX		= 0x04,
	eDRVPDMA_I2S_TX		= 0x05,
	eDRVPDMA_DAC0_TX	= 0x06,
	eDRVPDMA_DAC1_TX	= 0x07,
	eDRVPDMA_SPI2_TX	= 0x08,
	eDRVPDMA_TMR0_CH0	= 0x09,
	eDRVPDMA_TMR0_CH1	= 0x0A,
	eDRVPDMA_TMR1_CH0	= 0x0B,
	eDRVPDMA_TMR1_CH1	= 0x0C,

	eDRVPDMA_SPI0_RX	= 0x10,
	eDRVPDMA_SPI1_RX	= 0x11,
	eDRVPDMA_UART0_RX	= 0x12,
	eDRVPDMA_UART1_RX	= 0x13,
	eDRVPDMA_USB_RX		= 0x14,
	eDRVPDMA_I2S_RX		= 0x15,
	eDRVPDMA_ADC		= 0x16,
	eDRVPDMA_SPI2_RX	= 0x18,
	eDRVPDMA_PWM_RX0	= 0x19,
	eDRVPDMA_PWM_RX2	= 0x1A,
	eDRVPDMA_PWM_RX4	= 0x1B,
	eDRVPDMA_PWM_RX6	= 0x1C,

	eDRVPDMA_NONE		= 0x1F

}E_DRVPDMA_APB_DEVICE;



typedef enum										/* PDMA APB Read/Write */
{
	eDRVPDMA_READ_APB	= 0,    
	eDRVPDMA_WRITE_APB  = 1
}E_DRVPDMA_APB_RW;


typedef enum										/* PDMA Operation Mode */
{
	eDRVPDMA_MODE_MEM2MEM	= 0,
	eDRVPDMA_MODE_APB2MEM	= 1,
	eDRVPDMA_MODE_MEM2APB	= 2
}E_DRVPDMA_MODE;									


typedef struct 								    	/* PDMA Source/ Destination Transfer Selection structure */
{
    uint32_t u32Addr;							   	/* Start Address */
    E_DRVPDMA_DIRECTION_SELECT eAddrDirection;		/* Transfer direction */
}STR_DRVPDMA_TRANSFER_SEL_T;


typedef struct DRVPDMA_STRUCT						/* PDMA structure */
{
    STR_DRVPDMA_TRANSFER_SEL_T 	sSrcCtrl;    	    /* Source Control */
	STR_DRVPDMA_TRANSFER_SEL_T 	sDestCtrl;		    /* Destination Control */
    E_DRVPDMA_TRANSFER_WIDTH  	u8TransWidth;		/* Transfer Width */
    E_DRVPDMA_MODE 			  	u8Mode;				/* Operation Mode */
	int32_t              	  	i32ByteCnt;			/* Byte Count */
    uint8_t						bIsTimeOutEnable;	/* Enable Timeout function */
}STR_PDMA_T;


typedef enum										/* PDMA Operation Mode */
{
	eDRVPDMA_VDMA_INCREMENTED	= 0,
	eDRVPDMA_VDMA_DECREMENTED	= 1
}E_DRVPDMA_VDMA_DIR_SELECT;									

typedef struct DRVVDMA_STRUCT						/* PDMA structure */
{
	E_DRVPDMA_VDMA_DIR_SELECT	u8Dir;
    uint32_t					u32SrcAddr;			/* Start Address */
    uint32_t					u32DestAddr;		/* Start Address */
	int32_t						i32ByteCnt;			/* Byte Count */
	uint16_t					u16Stbc;			/* Stride Transfer Byte Count */
	uint16_t					u16Sastobl;			/* Source Address Stride Offset Byte Length */
	uint16_t					u16Dastobl;			/* Destination Address Stride Offset Byte Length */
    uint8_t						bIsStrideEnable;	/* Stride Mode Enable */
}STR_VDMA_T;


/*---------------------------------------------------------------------------------------------------------*/
/* APIs declaration                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/

void DrvPDMA_Init(void);
void DrvPDMA_Close(void);
void DrvPDMA_ClearIntFlag(E_DRVPDMA_CHANNEL_INDEX eChannel, E_DRVPDMA_INT_FLAG eIntFlag);

int32_t DrvPDMA_IsCHBusy(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_Open(E_DRVPDMA_CHANNEL_INDEX sChannel, STR_PDMA_T *sParam);
//int32_t DrvPDMA_IsEnabledCH(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_GetTransferLength(E_DRVPDMA_CHANNEL_INDEX eChannel,uint32_t* pu32TransferLength);
int32_t DrvPDMA_SetAPBTransferWidth(E_DRVPDMA_CHANNEL_INDEX eChannel,E_DRVPDMA_TRANSFER_WIDTH eTransferWidth);
int32_t DrvPDMA_GetAPBTransferWidth(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_GetCHForAPBDevice(E_DRVPDMA_APB_DEVICE eDevice,E_DRVPDMA_APB_RW eRWAPB);
//int32_t DrvPDMA_GetWrapIntType(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_CHSoftwareReset(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_CHEnableTransfer(E_DRVPDMA_CHANNEL_INDEX eChannel);
int32_t DrvPDMA_EnableInt(E_DRVPDMA_CHANNEL_INDEX eChannel,E_DRVPDMA_INT_ENABLE eIntSource);
int32_t DrvPDMA_DisableInt(E_DRVPDMA_CHANNEL_INDEX eChannel, E_DRVPDMA_INT_ENABLE eIntSource);
int32_t DrvPDMA_IsIntEnabled(E_DRVPDMA_CHANNEL_INDEX eChannel,E_DRVPDMA_INT_ENABLE eIntSource);
int32_t DrvPDMA_PollInt(E_DRVPDMA_CHANNEL_INDEX eChannel,  E_DRVPDMA_INT_FLAG eIntFlag);

uint32_t DrvPDMA_GetCurrentSourceAddr(E_DRVPDMA_CHANNEL_INDEX eChannel);
uint32_t DrvPDMA_GetCurrentDestAddr(E_DRVPDMA_CHANNEL_INDEX eChannel);
uint32_t DrvPDMA_GetRemainTransferCount(E_DRVPDMA_CHANNEL_INDEX eChannel);
uint32_t DrvPDMA_GetInternalBufPointer(E_DRVPDMA_CHANNEL_INDEX eChannel);
uint32_t DrvPDMA_GetSharedBufData(E_DRVPDMA_CHANNEL_INDEX eChannel);

int32_t DrvPDMA_SetTransferSetting(E_DRVPDMA_CHANNEL_INDEX eChannel, 
	uint32_t u32SrcAddr, uint32_t u32DestAddr, uint32_t u32TransferLength);

int32_t DrvPDMA_SetCHForAPBDevice(
    E_DRVPDMA_CHANNEL_INDEX eChannel, 
    E_DRVPDMA_APB_DEVICE eDevice,
    E_DRVPDMA_APB_RW eRWAPB    
);

int32_t DrvPDMA_InstallCallBack(
    E_DRVPDMA_CHANNEL_INDEX eChannel, 
    E_DRVPDMA_INT_ENABLE eIntSource,
	PFN_DRVPDMA_CALLBACK pfncallback
);

int32_t DrvPDMA_UnInstallCallBack(
    E_DRVPDMA_CHANNEL_INDEX eChannel, 
    E_DRVPDMA_INT_ENABLE eIntSource
);


int32_t DrvPDMA_GetCHForAPBDevice(E_DRVPDMA_APB_DEVICE eDevice,E_DRVPDMA_APB_RW eRWAPB);
int32_t DrvPDMA_GetVersion(void);

int32_t DrvPDMA_OpenVDMA(STR_VDMA_T *sParam);
int32_t DrvPDMA_SetVDMATransferSetting(uint32_t u32Stbc, uint32_t u32Sastobl, uint32_t u32Dastobl);

int32_t PDMA_SetTimeOutCount(E_DRVPDMA_CHANNEL_INDEX eChannel, uint32_t Count);


#ifdef  __cplusplus
}
#endif

#endif	// DRVPDMA_H














