/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef _NANO1XX_PWM_H
#define _NANO1XX_PWM_H

/*---------------------------------------------------------------------------------------------------------*/
/* Includes of system headers                                                                              */
/*---------------------------------------------------------------------------------------------------------*/

#include "nano1xx.h"

#ifdef  __cplusplus
extern "C"
{
#endif

/*---------------------------------------------------------------------------------------------------------*/
/*  Define Version number								                                                   */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_MAJOR_NUM	1
#define DRVPWM_MINOR_NUM	00
#define DRVPWM_BUILD_NUM	0

/*---------------------------------------------------------------------------------------------------------*/
/*  Version define with SysInfra				                                                           */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_VERSION_NUM     _SYSINFRA_VERSION(DRVPWM_MAJOR_NUM, DRVPWM_MINOR_NUM, DRVPWM_BUILD_NUM)
							   
/*---------------------------------------------------------------------------------------------------------*/
/*  Define Error Code									                                                   */
/*---------------------------------------------------------------------------------------------------------*/
// E_DRVPWM_ERR_ARGUMENT  		Incorrect Argument
// E_DRVPWM_ERR_NO_SUPPORT		Does not support this function
#define E_DRVPWM_ERR_ARGUMENT   _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVPWM, 1)
#define E_DRVPWM_ERR_NO_SUPPORT _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVPWM, 2)

/** Max valid channel number. Each set of PWM in NANO1xx supports four channels, 0~3. */
#define PWM_MAX_CH      3

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Timer 								                                                               */
/*---------------------------------------------------------------------------------------------------------*/
#define	DRVPWM_TIMER0   0x00
#define	DRVPWM_TIMER1   0x01
#define	DRVPWM_TIMER2   0x02
#define	DRVPWM_TIMER3   0x03
#define	DRVPWM_TIMER4   0x04
#define	DRVPWM_TIMER5   0x05
#define	DRVPWM_TIMER6   0x06
#define	DRVPWM_TIMER7   0x07

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Capture								                                                               */
/*---------------------------------------------------------------------------------------------------------*/
#define	DRVPWM_CAP0     0x10
#define	DRVPWM_CAP1	    0x11
#define	DRVPWM_CAP2     0x12
#define	DRVPWM_CAP3     0x13
#define	DRVPWM_CAP4     0x14
#define	DRVPWM_CAP5	    0x15
#define	DRVPWM_CAP6     0x16
#define	DRVPWM_CAP7     0x17

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Capture Interrupt Enable/Disable Flag                                                              */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_CAP_RISING_INT	1
#define DRVPWM_CAP_FALLING_INT	2
#define DRVPWM_CAP_ALL_INT		3

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Capture Latched Indicator				                                                           */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_CAP_INT_FLAG	1
#define DRVPWM_CAP_RISING_FLAG	2
#define DRVPWM_CAP_FALLING_FLAG	4

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Clcok Selector						                                                               */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_CLOCK_DIV_1    4
#define DRVPWM_CLOCK_DIV_2    0
#define DRVPWM_CLOCK_DIV_4    1
#define DRVPWM_CLOCK_DIV_8    2
#define DRVPWM_CLOCK_DIV_16   3

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Clcok Source Selector						                                                       */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_EXT_12M    	  0
#define DRVPWM_EXT_32K        1
#define DRVPWM_HCLK           2
#define DRVPWM_INTERNAL_22M   3

/*---------------------------------------------------------------------------------------------------------*/
/*  PWM Mode Selector						                                                               */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVPWM_AUTO_RELOAD_MODE	1
#define DRVPWM_ONE_SHOT_MODE	0

/*---------------------------------------------------------------------------------------------------------*/
/* Define PWM Time Data Struct                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
typedef struct
{
	uint8_t	  u8Mode;
	uint8_t	  u8HighPulseRatio;
	uint8_t	  u8ClockSelector;
	uint8_t	  u8PreScale;
	uint32_t  u32Frequency;	
	uint32_t  u32Duty;
 	int32_t	  i32Inverter;
}S_DRVPWM_TIME_DATA_T;

/*---------------------------------------------------------------------------------------------------------*/
/* Define PWM Call back funvtion Data Struct                                                               */
/*---------------------------------------------------------------------------------------------------------*/
typedef void (*PFN_DRVPWM_CALLBACK)(void);

/*---------------------------------------------------------------------------------------------------------*/
/* Define PWM Call back function Data Struct                                                               */
/*---------------------------------------------------------------------------------------------------------*/
typedef struct
{
    PFN_DRVPWM_CALLBACK pfnPWMCallBack[4];    
    PFN_DRVPWM_CALLBACK pfnCAPCallBack[4];        
   
}S_DRVPWM_CALLBACK_T;

/**
 * Indicates PWM PDMA capture order if both rising and falling capture is enabled
 */
typedef enum {
    PWM_FALLING_FIRST = 0,    ///< Falling event capture first
    PWM_RISING_FIRST = 1      ///< Rising event capture first
}E_PWM_CAP_ORDER;

/**
 * Indicates PWM interrupts to be enabled while generator function selected
 * If PDMA is enabled, transfer enabled capture event
 */
typedef enum {
	PWM_CAP_RISING = 1,   ///< Enable rising latch interrupt
    PWM_CAP_FALLING = 2,  ///< Enable falling latch interrupt  
    PWM_CAP_BOTH = 3      ///< Enable both rising and falling interrupt
}E_PWM_CAP_MODE;

/** Indicates PWM capture counter reload event */
typedef enum {
	PWM_RELOAD_ZERO = 0,	   ///< PWM counter reload after reach zero
	PWM_RELOAD_RISING = 1,	   ///< PWM counter reload after rising capture event
	PWM_RELOAD_FALLING = 2,	   ///< PWM counter reload after falling capture event 
	PWM_RELOAD_BOTH =3	       ///< PWM counter reload after both capture event
}E_PWM_RELOAD;

/*---------------------------------------------------------------------------------------------------------*/
/* Define PWM functions prototype                                                                          */
/*---------------------------------------------------------------------------------------------------------*/

void	 DrvPWM_ClearCaptureIntStatus(uint8_t u8Capture, uint8_t u8IntType);
void	 DrvPWM_ClearInt(uint8_t u8Timer);
void	 DrvPWM_Close(uint8_t u8Timer);
void	 DrvPWM_DisableInt(uint8_t u8Timer);

void	 DrvPWM_Enable(uint8_t u8Timer);
void     DrvPWM_Disable(uint8_t u8Timer);
void	 DrvPWM_EnableDeadZone(uint8_t u8Timer, uint8_t u8Length); 
void	 DrvPWM_DisableDeadZone(uint8_t u8Timer); 
void	 DrvPWM_EnableInt(uint8_t u8Timer, uint8_t u8Int, PFN_DRVPWM_CALLBACK pfncallback);

int32_t	 DrvPWM_GetCaptureIntStatus(uint8_t u8Capture, uint8_t u8IntType);
uint32_t DrvPWM_GetFallingCounter(uint8_t u8Capture);  
int32_t	 DrvPWM_GetIntFlag(uint8_t u8Timer);
uint32_t DrvPWM_GetRisingCounter(uint8_t u8Capture);  
uint32_t DrvPWM_GetTimerCounter(uint8_t u8Timer);  
uint32_t DrvPWM_GetVersion (void);

void	 DrvPWM_Open(uint8_t u8Timer); 
int32_t  DrvPWM_SelectClearLatchFlagOption(int32_t i32option);
void 	 DrvPWM_SelectClockSource(uint8_t u8Timer, uint8_t u8ClockSourceSelector);
uint32_t DrvPWM_SetTimerClk(uint8_t u8Timer, S_DRVPWM_TIME_DATA_T *sPt);
void DrvPWM_SetTimerCounter(uint8_t u8Timer, uint32_t cn, uint16_t cm);
void	 DrvPWM_SetTimerIO(uint8_t u8Timer, int32_t i32Enable);

int32_t DrvPWM_DisablePdma(uint8_t u8Timer);
int32_t DrvPWM_EnablePdma(uint8_t u8Timer, E_PWM_CAP_ORDER order, E_PWM_CAP_MODE mode);
int32_t DrvPWM_SetCaptureAttribute(uint8_t u8Capture, uint8_t inv, uint8_t cas, E_PWM_RELOAD reload);

#ifdef  __cplusplus
}
#endif

#endif  //_NANO1XX_PWM_H



