/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef __NANO1XX_TIMER_H__
#define __NANO1XX_TIMER_H__

#include "nano1xx.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/*  Define Version number                                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVTIMER_MAJOR_NUM      1
#define DRVTIMER_MINOR_NUM      00
#define DRVTIMER_BUILD_NUM      000
#define DRVTIMER_VERSION_NUM    _SYSINFRA_VERSION(DRVTIMER_MAJOR_NUM, DRVTIMER_MINOR_NUM, DRVTIMER_BUILD_NUM)

/*---------------------------------------------------------------------------------------------------------*/
/* Define Error Code                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
#define E_DRVTIMER_CHANNEL          _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTIMER, 1)  /* Invalid channel     */
#define E_DRVTIMER_CLOCK_RATE       _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTIMER, 2)  /* Invalid clock rate  */
#define E_DRVTIMER_EIO              _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTIMER, 3)  /* Timer IO error      */
#define E_DRVTIMER_EVENT_FULL       _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTIMER, 4)  /* Event Full error    */
#define E_DRVWDT_CMD                _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTIMER, 5)  /* Invalid CMD         */

#define TIMER_TRIGGER_PDMA  1
#define TIMER_TRIGGER_ADC   2
#define TIMER_TRIGGER_DAC   4

#define TIMER_MAX_VAL       0xFFFFFF
#define TIMER_TOTAL_CHANNEL 4

/*---------------------------------------------------------------------------------------------------------*/
/* Global interface variables declarations                                                                 */                                                                            
/*---------------------------------------------------------------------------------------------------------*/
typedef void (*TIMER_CALLBACK)(uint32_t data);  /* function pointer */
typedef void (*WDT_CALLBACK)(uint32_t data);    /* function pointer */

/*---------------------------------------------------------------------------------------------------------*/
/* Define TIME EVENT STRUCT                                                                                */
/*---------------------------------------------------------------------------------------------------------*/
typedef struct timeEvent_t
{
    int32_t              active;
    int32_t              initTick;
    int32_t              curTick;
    TIMER_CALLBACK       funPtr;
    uint32_t             transParam;
} TIMER_EVENT_T;

/*---------------------------------------------------------------------------------------------------------*/
/* Define TIMER Channel                                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum {
    E_TMR0  =   0 ,
    E_TMR1  =   1 ,
    E_TMR2  =   2 ,
    E_TMR3  =   3        
} E_TIMER_CHANNEL;

/*---------------------------------------------------------------------------------------------------------*/
/* Define TIMER OPREATION MODE                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum{                   
    E_ONESHOT_MODE        = 0,
    E_PERIODIC_MODE     = 1,
    E_TOGGLE_MODE       = 2, 
    E_CONTINUOUS_MODE   = 3     
} E_TIMER_OPMODE ;

typedef enum {
    E_TIMER_TMR_INT = 1,     ///< Timer counter interrupt
    E_TIMER_TCAP_INT = 2,    ///< Timer capture interrupt
    E_TIMER_BOTH_INT = 3     ///< Both interrupt
}E_TIMER_INT;

typedef enum {
    E_TIMER_FALLING_EDGE = 0,
    E_TIMER_RISING_EDGE = 1,
    E_TIMER_FALLING_RISING_EDGE = 2,
    E_TIMER_RISING_FALLING_EDGE = 3,    
}E_TIMER_CAP_EDGE;

typedef enum {
    E_TIMER_CAP_FREE_COUNTING = 0,
    E_TIMER_CAP_TRIGGER_COUNTING = 1,   
}E_TIMER_CAP_MODE;

typedef enum {
    E_TIMER_EVENT_EDGE_FALLING = 0,
    E_TIMER_EVENT_EDGE_RISING = 1
}E_TIMER_EVENT_EDGE;

typedef enum {
    E_TIMER_TRIGGER_TMR = 0,
    E_TIMER_TRIGGER_TCAP = 1    
}E_TIMER_TRIGGER_SRC;

/*---------------------------------------------------------------------------------------------------------*/
/* Define WDT Ioctl Command                                                                                */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum {
    E_WDT_IOC_START_TIMER       = 0,
    E_WDT_IOC_STOP_TIMER        = 1,
    E_WDT_IOC_ENABLE_INT        = 2,
    E_WDT_IOC_DISABLE_INT       = 3,
    E_WDT_IOC_ENABLE_WAKEUP     = 4, 
    E_WDT_IOC_DISABLE_WAKEUP    = 5, 
    E_WDT_IOC_RESET_TIMER       = 6,
    E_WDT_IOC_ENABLE_RESET_FUNC = 7,
    E_WDT_IOC_DISABLE_RESET_FUNC= 8,
    E_WDT_IOC_SET_INTERVAL      = 9
} E_WDT_CMD; 

/*---------------------------------------------------------------------------------------------------------*/
/* Define WDT INTERVAL                                                                                     */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum{
    E_WDT_LEVEL0      = 0,         /* 2^4 WDT_CLK                                                              */
    E_WDT_LEVEL1      = 1,         /* 2^6 WDT_CLK                                                            */
    E_WDT_LEVEL2      = 2,         /* 2^8 WDT_CLK                                                            */ 
    E_WDT_LEVEL3      = 3,         /* 2^10 WDT_CLK                                                           */
    E_WDT_LEVEL4      = 4,         /* 2^12 WDT_CLK                                                           */
    E_WDT_LEVEL5      = 5,         /* 2^14 WDT_CLK                                                           */
    E_WDT_LEVEL6      = 6,         /* 2^16 WDT_CLK                                                           */
    E_WDT_LEVEL7      = 7          /* 2^18 WDT_CLK                                                           */
} E_WDT_INTERVAL;




/*---------------------------------------------------------------------------------------------------------*/
/* Define Function Prototype                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void DrvTIMER_Init(void);
int32_t DrvTIMER_Open(E_TIMER_CHANNEL ch, uint32_t uTicksPerSecond, E_TIMER_OPMODE op_mode);
int32_t DrvTIMER_Close(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_SetTimerEvent(E_TIMER_CHANNEL ch, uint32_t uInterruptTicks, TIMER_CALLBACK pTimerCallback, uint32_t parameter);
void DrvTIMER_ClearTimerEvent(E_TIMER_CHANNEL ch, uint32_t uTimerEventNo);
int32_t DrvTIMER_EnableInt(E_TIMER_CHANNEL ch, E_TIMER_INT eInterrupt);
int32_t DrvTIMER_DisableInt(E_TIMER_CHANNEL ch, E_TIMER_INT eInterrupt);
int32_t DrvTIMER_GetIntFlag(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_ClearIntFlag(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_Start(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_Stop(E_TIMER_CHANNEL ch);
uint32_t DrvTIMER_GetIntTicks(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_ResetIntTicks(E_TIMER_CHANNEL ch);
void DrvTIMER_Delay(E_TIMER_CHANNEL ch, uint32_t uTicks);
void DrvTIMER_SetEXTClockFreq(uint32_t u32ClockFreq);
int32_t DrvTIMER_OpenCounter(E_TIMER_CHANNEL ch, uint32_t uCounterBoundary, E_TIMER_OPMODE op_mode);
uint32_t DrvTIMER_GetCounters(E_TIMER_CHANNEL ch);
uint32_t DrvTIMER_GetVersion(void);
int32_t DrvTIMER_IsWakeUpSource(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_StartCounter(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_Reset(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_EnableCaptureMode(E_TIMER_CHANNEL ch, E_TIMER_CAP_MODE mode, E_TIMER_CAP_EDGE edge);
int32_t DrvTIMER_Stop(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_SetPrescale(E_TIMER_CHANNEL ch, uint8_t prescale);
int32_t DrvTIMER_EnableInterTimerTriggerMode(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_DisableInterTimerTriggerMode(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_EnableCounterResetMode(E_TIMER_CHANNEL ch, E_TIMER_CAP_EDGE edge);
int32_t DrvTIMER_DisableCounterResetMode(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_DisableCaptureMode(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_EnableExternalEventMode(E_TIMER_CHANNEL ch, E_TIMER_EVENT_EDGE edge);
int32_t DrvTIMER_DisableExternalEventMode(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_EnableWakeUp(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_DisableWakeUp(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_GetCaptureData(E_TIMER_CHANNEL ch);
int32_t DrvTIMER_SetCompareValue(E_TIMER_CHANNEL ch, uint32_t value);
int32_t DrvTIMER_ConfigTriggerEvent(E_TIMER_CHANNEL ch, uint8_t event, E_TIMER_TRIGGER_SRC src);
int32_t DrvTIMER_IsActive(E_TIMER_CHANNEL ch);///


void DrvWDT_Open(E_WDT_INTERVAL WDTlevel);
void DrvWDT_Close(void);
void DrvWDT_InstallISR(WDT_CALLBACK pvWDTISR, uint32_t param);
int32_t DrvWDT_Ioctl(E_WDT_CMD uWDTCmd, uint32_t uArgument);

#endif  // __NANO1XX_TIMER_H__


