/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

#ifndef _NANO100_UDC_H_
#define _NANO100_UDC_H_

#include "Driver\DrvUSB.h"

//======================================================
// USB Control
//======================================================
// USB STATE
#define	USB_STATE_DETACHED			0x00
#define	USB_STATE_ATTACHED			0x01
#define	USB_STATE_DEFAULT			0x11
#define	USB_STATE_ADDRESS			0x31
#define	USB_STATE_CONFIGURED		0x71

#define	USB_STATE_FLAG_DEFAULT		0x10
#define	USB_STATE_FLAG_ADDRESS		0x20
#define	USB_STATE_FLAG_CONFIGURED	0x40
#define	USB_STATE_FLAG_SUSPENDED	0x80

// FLAG
#define	FLAG_OUT_ACK		0x01
#define	FLAG_SET_ADDRESS	0x02
#define	FLAG_SET_CONFIG		0x04
#define	FLAG_SET_INTERFACE	0x08
#define	FLAG_GET_MAX_LUN	0x10

// Max Packet Size
#define	MAX_PACKET_SIZE0	64
#define	MAX_PACKET_SIZE		64

// SRAM
#define	BUF_SETUP		0xF8
#define	BUF_CTRL		0x00
#define	BUF_BULK0		(BUF_CTRL+MAX_PACKET_SIZE0)
#define	BUF_BULK1		(BUF_BULK0+MAX_PACKET_SIZE)

// CFG
#define	CFG0_SETTING	(EPTCFG_CSTALL | EPTCFG_IN  | 0)	/* EP address 0, control IN  */
#define	CFG1_SETTING	(EPTCFG_CSTALL | EPTCFG_OUT | 0)	/* EP address 0, control OUT */
#define	CFG2_SETTING	(EPTCFG_IN  | 2)					/* EP address 2 */
#define	CFG3_SETTING	(EPTCFG_OUT | 3)					/* EP address 3 */

// BUF
#define	BUF0_SETTING	BUF_CTRL
#define	BUF1_SETTING	BUF_CTRL
#define	BUF2_SETTING	BUF_BULK1
#define	BUF3_SETTING	BUF_BULK0
						
#define	g_au8UsbSetup	((uint8_t *)(USBD_BASE+0x100+BUF_SETUP))
#define	g_au8UsbCtrl	((uint8_t *)(USBD_BASE+0x100+BUF_CTRL))
#define	g_au8UsbBulk1	((uint8_t *)(USBD_BASE+0x100+BUF_BULK1))
#define	g_au8UsbBulk0	((uint8_t *)(USBD_BASE+0x100+BUF_BULK0))

//======================================================
#define	BULK_ONLY_MASS_STORAGE_RESET	0xFF
#define	GET_MAX_LUN						0xFE

//======================================================
#define	BULK_CBW	0x00
#define	BULK_IN		0x01
#define	BULK_OUT	0x02
#define	BULK_CSW	0x04
#define BULK_NORMAL	0xFF

#define	CBW_SIGNATURE	0x43425355
#define	CSW_SIGNATURE	0x53425355

//======================================================
#define	UFI_TEST_UNIT_READY								0x00
#define	UFI_REQUEST_SENSE								0x03
#define	UFI_INQUIRY										0x12
#define	UFI_MODE_SELECT_6								0x15
#define	UFI_MODE_SENSE_6								0x1A
#define	UFI_START_STOP									0x1B
#define	UFI_PREVENT_ALLOW_MEDIUM_REMOVAL				0x1E
#define	UFI_READ_FORMAT_CAPACITY						0x23
#define	UFI_READ_CAPACITY								0x25
#define	UFI_READ_10										0x28
#define	UFI_WRITE_10									0x2A
#define	UFI_VERIFY_10									0x2F
#define	UFI_MODE_SELECT_10								0x55
#define	UFI_MODE_SENSE_10								0x5A

//======================================================
/* for mass storage */
struct CBW
{
	uint32_t	dCBWSignature;
	uint32_t	dCBWTag;
	uint32_t	dCBWDataTransferLength;
	uint8_t		bmCBWFlags;
	uint8_t		bCBWLUN;
	uint8_t		bCBWCBLength;
	uint8_t		u8OPCode;
	uint8_t		u8LUN;
	uint8_t		au8Data[14];
};

struct CSW
{
	uint32_t	dCSWSignature;
	uint32_t	dCSWTag;
	uint32_t	dCSWDataResidue;
	uint8_t		bCSWStatus;
};

typedef struct Disk_Par_Inf 
{
	uint32_t	partition_size,
				data_location,
				bpb_location,
				fat_location,
				rootdir_location,
				free_size;
	uint16_t	rootdirentryno,
				totalcluster,
				NumCyl;
	uint8_t		NumHead,
				NumSector,
				capacity,
				fatcopyno,
				fatsectorno,
				fat16flg;
} Disk_Par_Info;

/* extern flags */
extern uint8_t g_u8BulkState;
extern int32_t udcOnLine;
extern int32_t g_bIsFirstRead;
extern uint32_t usbDataReady;
extern uint8_t g_u8UsbState;

/* extern functions */
void ReqSenCommand(void);
void RdFmtCapCommand(void);
void RdCurCapCommand(void);
uint8_t udcFlashInit(void);

void udcInit(void);  
void udcMassBulk(void);

#endif
