
#ifndef FMC_H
#define FMC_H

//#define FLASH64K
#undef FMC_BASE
#undef outp32
#undef inp32

#define outp32(port,value)	(*((volatile unsigned int *)(port))=(value))
#define inp32(port)			(*((volatile unsigned int *)(port)))

#define FMC_BASE        0x5000C000UL		
 
#define FISPCON         (FMC_BASE+0x000)			
#define ISPADR         (FMC_BASE+0x004)		
#define ISPDAT         (FMC_BASE+0x008)			
#define ISPCMD         (FMC_BASE+0x00C)			
#define ISPTRG         (FMC_BASE+0x010)			
#define DFBADDR        (FMC_BASE+0x014)
#define FATCON         (FMC_BASE+0x018)			
#define ICPCON         (FMC_BASE+0x01C)			
#define RMPCON         (FMC_BASE+0x020)			

#define Config0         0x00300000
#define Config1         0x00300004

// For LD ROM
#define LDROM_BASE      0x00100000
#define LDROM_SIZE      0x00001000
#define LDROM_END       0x00101000

#define ISPGO           0x01
#define ISPFF           0x00000040


#define ISP_Read        0x00
#define ISP_Program     0x21
#define ISP_PageErase   0x22
#define ISP_ReadCID	    0x0B			/*!<ISP Command - Read CID */
#define ISP_ReadDID	    0x0C			/*!<ISP Command - Read DID */
#define ISP_ReadUID	    0x04			/*!<ISP Command - Read UID */
#define ISP_VecRemap	0x2E			/*!<ISP Command - Vector Page Re-Map */


/*---------------------------------------------------------------------------------------------------------*/
/* Define parameter                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PAGE_SIZE                      0x00000200     /* Page size                                         */
//#define PAGE_NUM                       32             /* Total page number                                 */

typedef int (*FMC_Write_t)(unsigned int address, unsigned int data);
typedef	int (*FMC_Read_t)(unsigned int address, unsigned int * data);
typedef	int (*FMC_Erase_t)(unsigned int address);
typedef int32_t (*FMC_SetVectorPage_t)(uint32_t u32addr);
typedef int32_t (*FMC_ReadCID_t)(uint32_t * data);
typedef	int32_t (*FMC_ReadDID_t)(uint32_t * data);
typedef	int32_t (*FMC_ReadPID_t)(uint32_t *data);
typedef	int32_t (*FMC_ReadUID_t)(int index, uint32_t *data);
typedef	void (*ReadData_t)(unsigned int addr_start, unsigned int addr_end, unsigned int* data);    // Read data from flash
typedef	void (*WriteData_t)(unsigned int addr_start, unsigned int addr_end, unsigned int *data);  // Write data into flash
typedef	void (*EraseAP_t)(unsigned int addr_start, unsigned int addr_end);
typedef	void (*UpdateConfig_t)(unsigned int *data, unsigned int *res);
typedef	void (*GetDataFlashInfo_t)(uint32_t apromsize, uint32_t *addr, uint32_t *size);


struct _FMC{	
	FMC_Read_t FMC_Read;
	FMC_Write_t FMC_Write;
	FMC_Erase_t FMC_Erase;
	FMC_SetVectorPage_t FMC_SetVectorPage;
	FMC_ReadCID_t FMC_ReadCID;
	FMC_ReadDID_t FMC_ReadDID;
	FMC_ReadPID_t FMC_ReadPID;
	FMC_ReadUID_t FMC_ReadUID;
	ReadData_t ReadData;
	WriteData_t WriteData;
	EraseAP_t EraseAP;
	UpdateConfig_t UpdateConfig;
	GetDataFlashInfo_t GetDataFlashInfo;
};

#define FMC_Write			gFMC.FMC_Write
#define FMC_Read			gFMC.FMC_Read
#define FMC_Erase			gFMC.FMC_Erase
#define FMC_SetVectorPage	gFMC.FMC_SetVectorPage
#define FMC_ReadCID			gFMC.FMC_ReadCID
#define FMC_ReadDID			gFMC.FMC_ReadDID
#define FMC_ReadPID			gFMC.FMC_ReadPID
#define FMC_ReadUID			gFMC.FMC_ReadUID
#define ReadData			gFMC.ReadData
#define WriteData			gFMC.WriteData
#define EraseAP				gFMC.EraseAP
#define UpdateConfig		gFMC.UpdateConfig
#define GetDataFlashInfo	gFMC.GetDataFlashInfo

#endif

