/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "NANO1xx.h"

#include "Driver/DrvUSB.h"
#include "Driver/DrvGPIO.h"
#include "USB/HIDDevice.h"
#include "USB/USBInfra.h"
#include "FMC.h"
#ifndef UART_ONLY
#include "HIDDevice.c"
#endif
//#define SUPPORT_WRITECKSUM
#define USING_AUTODETECT //using autodetect for UART download
//#define USING_RS485 //default using GPD13 act as Rx/Tx switch for RS485 tranceiver

#define CMD_UPDATE_APROM	0x000000A0
#define CMD_UPDATE_CONFIG	0x000000A1
#define CMD_READ_CONFIG		0x000000A2
#define CMD_ERASE_ALL		0x000000A3
#define CMD_SYNC_PACKNO		0x000000A4
#define CMD_GET_FWVER		0x000000A6
#define CMD_SET_APPINFO     0x000000A7
#define CMD_GET_APPINFO     0x000000A8
#define CMD_RUN_APROM		0x000000AB
#define CMD_RUN_LDROM		0x000000AC
#define CMD_RESET			0x000000AD
#define CMD_CONNECT			0x000000AE
#define CMD_DISCONNECT		0x000000AF

#define CMD_GET_DEVICEID	0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2

typedef void (FUNC_PTR)(void);

static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus;
__align(4) static uint8_t uart_rcvbuf[64];
__align(4) static uint8_t uart_sendbuf[64];
__align(4) static uint8_t aprom_buf[PAGE_SIZE];
BOOL bUsbDataReady, bUartDataReady, bUsingUART;
BOOL bUsbInReady, bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid, g_timecnt;
#ifdef SUPPORT_WRITECKSUM
static uint32_t g_ckbase = (0x20000 - 8);
static void CheckCksumBase(void);
#endif

static UART_TypeDef	*g_pUART = UART0;
static IRQn_Type	g_UARTIRQ = UART0_IRQn;
static uint32_t gAppBase;
static FUNC_PTR  		*gFunc;


void Delay(uint32_t delayCnt);
void my_memcpy(void *dest, void *src, int32_t size);
extern void WordsCpy(void *dest, void *src, int32_t size);
void SysTimerDelay(uint32_t us);


struct _FMC gFMC = {	
	(FMC_Read_t)0x00100001,
	(FMC_Write_t)0x00100081,
	(FMC_Erase_t)0x00100101,
	(FMC_SetVectorPage_t)0x00100181,
	(FMC_ReadCID_t)0x00100201,
	(FMC_ReadDID_t)0x00100281,
	(FMC_ReadPID_t)0x00100301,
	(FMC_ReadUID_t)0x00100381,
	(ReadData_t)0x00100401,
	(WriteData_t)0x00100481,
	(EraseAP_t)0x00100501,
	(UpdateConfig_t)0x00100581,
	(GetDataFlashInfo_t)0x00100601
};  
void NMI_Handler(void)
{   
	outpw(&GCR->BODSTS, 0xE);
}   
    
    
/*  
* send data for UART
*/  
static __inline void PutString()
{   
	int i;
    
#ifdef USING_RS485	
	GPIOD->DOUT |= 0x2000;//transmit mode
	SysTimerDelay(13);//we send 64 byte once, if send 1 bytes only, please change it to 60
#endif
	for(i = 0; i<HID_MAX_PACKET_SIZE_EP1; i++)
	{
		while(g_pUART->FSR_BITS.TX_POINTER_F >= 14);
		//while(g_pUART->FSR.TX_EMPTY == 0);
		g_pUART->THR = uart_sendbuf[i];
	}
#ifdef USING_RS485
	while(g_pUART->FSR.TX_POINTER);
	SysTimerDelay(26);////we send 64 byte once, if send 1 bytes only, please change it to 110
	GPIOD->DOUT &= ~0x2000;//reveive mode
#endif
}   
    
/*  
* UART0/UART1 interrupt handler. handle receive only
*/  
void UART0_IRQHandler(void)//UART1_IRQHandler
{   
	/*----- Determine interrupt source -----*/
    
	if (inpw(&g_pUART->ISR) & 0x11) //RDA FIFO interrupt & RDA timeout interrupt 
   	{
   		while(((inpw(&g_pUART->FSR) & (0x2)) == 0) && (bufhead<64))//RX fifo not empty
			uart_rcvbuf[bufhead++] = inpw(&g_pUART->RBR);
	}
	
	if(bufhead == 64)
	{
		bUartDataReady = TRUE;
		bufhead = 0;
	}
}   
    
static __inline void UartInit(void)
{   
	if(g_pUART == UART0)
	{
    	/* Multi-Function Pin: Enable UART0:Tx Rx */
    	//GCR->PA_H_MFP_BITS.PA14	= 0x6;
    	//GCR->PA_H_MFP_BITS.PA15	= 0x6;
    	outpw(&GCR->PA_H_MFP, 0x66000000);
    
    	/* Configure GCR to reset UART0 */
    	GCR->IPRST_CTL2_BITS.UART0_RST = 1;
    	GCR->IPRST_CTL2_BITS.UART0_RST = 0;
    
    	/* Enable UART clock */
    	CLK->APBCLK_BITS.UART0_EN = 1;
	}
	else//UART1
	{
		//GCR->PA_H_MFP_BITS.PA14	= 0x6;
    	//GCR->PA_H_MFP_BITS.PA15	= 0x6;
    	outpw(&GCR->PA_H_MFP, 0x66000000);
    
    	/* Configure GCR to reset UART0 */
    	GCR->IPRST_CTL2_BITS.UART1_RST = 1;
    	GCR->IPRST_CTL2_BITS.UART1_RST = 0;
    
    	/* Enable UART clock */
    	CLK->APBCLK_BITS.UART1_EN = 1;
	}
    
	
#ifdef USING_RS485	
	GPIOD->PMD.PMD13 = 1;//OUT
	GPIOD->DOUT &= ~0x2000;//reveive mode
#endif
	
		
	
    /* Select UART clock source */
    CLK->CLKSEL1_BITS.UART_S = 3;//0:12M; 3:HIRC:12M
    
    /* Data format */
    //g_pUART->TLCTL.DATA_LEN = 3;
	//UART0->FCR.RFR = 1;//reset rx fifo
	//UART0->FCR.RFITL = 5;//46
	//outpw(&g_pUART->FCR, 0x52);//46
	outpw(&g_pUART->TLCTL, 0x303);//14
	g_pUART->TMCTL = 0x40;
    
    /* Configure the baud rate */
    //BaudRateCalculator(SystemCoreClock, 115200, &UART0->BAUD);
#if 1//12M
    *((__IO uint32_t *)&UART0->BAUD) = 0x00000067;//115200
    //*((__IO uint32_t *)&g_pUART->BAUD) = 0xB;//57600
     //*((__IO uint32_t *)&g_pUART->BAUD) = 0x3F0009C2;//4800
#else //22.1184M
	 *((__IO uint32_t *)&g_pUART->BAUD) = 0x2F00000A;//115200
	 //*((__IO uint32_t *)&UART0->BAUD) = 0x16;//57600
#endif
	
    NVIC_SetPriority (g_UARTIRQ, 2);
   	NVIC_EnableIRQ(g_UARTIRQ);
    
    //UART0->IER.RDA_IEN = 1;UART0->IER.TOC_IEN = 1;UART0->IER.TOC_EN = 1;//Enable Timeout Counter
	outpw(&g_pUART->IER, 0x11);
    
}   
    
#ifndef UART_ONLY
/*  
* USB interrupt handler
*/  
void USBD_IRQHandler(void)
{   
    //printf("USB relative events\n");
	USB_ParseEvent();
}   
    
/*  
* init HID usb
*/  
static void UsbHid(void)
{   
	HID_Open();	
	
    NVIC_SetPriority (USBD_IRQn, 2);//(1<<__NVIC_PRIO_BITS) - 2);
    NVIC_EnableIRQ(USBD_IRQn);
    
	/* Enable float-detection interrupt. */
	_DRVUSB_ENABLE_FLDET_INT();
    
	/* Enable USB-related interrupts. */
	_DRVUSB_ENABLE_WAKEUP();//enable usb wakeup function
	_DRVUSB_ENABLE_MISC_INT(INTEN_WAKEUP | INTEN_FLDET | INTEN_USBEVT | INTEN_BUSEVT);
	
}   
    
#endif
    
static uint16_t Checksum (unsigned char *buf, int len)
{   
    int i;
    uint16_t c;
    
    for (c=0, i=0; i < len; i++) {
        c += buf[i];
    }
    return (c);
}   
    
static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{   
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}   

__asm spChange(uint32_t _sp)
{
	MSR MSP, r0
	BX lr
}
    
static int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{   
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t	lcmd, packno, srclen, i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	
	volatile uint32_t appInfo, appLen;
	
	response = uart_sendbuf;
#ifndef  UART_ONLY
	if(bUSB)
		response = g_HID_sDevice.au8IntInBuffer;
#endif
	pSrc = buffer;
	srclen = len;
    
	lcmd = inpw(pSrc);
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));//read config
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;
    
	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	if((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;
	
    
	if((lcmd)&&(lcmd!=CMD_RESEND_PACKET))
		gcmd = lcmd;
    
	if(lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;//version 2.3
	//else if(lcmd == CMD_READ_CONFIG)
	//{
		//FMC_Read(Config0, (uint32_t*)(response+8));
		//FMC_Read(Config1, (uint32_t*)(response+12));
	//}
	else if(lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, GCR->PDID);
		goto out;
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		outpw(&GCR->RST_SRC, 3);//clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = gAppBase;
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			
		}
		else
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFE);//ISP disable
			outpw(&FMC->ISPCON, i);
			outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));
			/* Trap the CPU */
			while(1);
		}
    
		FMC_SetVectorPage(i);
		
		gFunc = (FUNC_PTR *)*(uint32_t *)(i + 4);
		spChange(*(uint32_t *)i);			
		gFunc();
	}
	else if(lcmd == CMD_CONNECT)
	{
		//g_connStatus = CONNECTED;
		g_packno = 1;
		goto out;
	}
	//else if(lcmd == CMD_DISCONNECT)
	//{
		//g_connStatus = DISCONNECTED;
	//}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		if(inpw(pSrc+24) != 0xEF09EA33)//deny to burn
			goto out;
			
		gAppBase = inpw(pSrc);//0xC800;

		if(gAppBase < 0x400)
			goto out;
		
		//the address of Reset_Handler doesn't beyond 0x100
		if((inpw(pSrc+12) < gAppBase) || (inpw(pSrc+12) - gAppBase > 0x100))
			goto out;
		
		GetDataFlashInfo(g_apromSize, &g_dataFlashAddr, &g_dataFlashSize);
		
		if(g_dataFlashSize != 0)
		{
			//appInfo = inpw(g_dataFlashAddr);
			if(FMC_Read(g_dataFlashAddr, &appInfo) < 0)
				goto out;
			if(FMC_Read(g_dataFlashAddr+4, &appLen) < 0)
				goto out;
			
			if(gAppBase == appInfo)
				goto out;
			if((appLen != 0xFFFFFFFF) && (gAppBase < (appInfo + appLen)) && (gAppBase > appInfo))//can't damage the orignal code
				goto out;
		}
		else
			goto out;
		
		if((regcnf0 & 0x4) && ((security == 0)||(lcmd == CMD_ERASE_ALL)))//erase APROM + data flash
		{
			//EraseAP(gAppBase, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
			if(gAppBase < appInfo)//app2 burn app1,don't erase app2
			{
				EraseAP(gAppBase, appInfo);//don't erase data flash	 and app2
				EraseAP(g_dataFlashAddr, g_dataFlashSize);//erase data flash
			}
			else
				EraseAP(gAppBase, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
		}
		else//don't erase data flash
		{
			if(gAppBase < appInfo)//app2 burn app1
				EraseAP(gAppBase, appInfo);//don't erase data flash	 and app2
			else
				EraseAP(gAppBase, g_dataFlashAddr);//don't erase data flash
		}
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		
		bUpdateApromCmd = TRUE;
	}
    
    
#ifdef SUPPORT_WRITECKSUM
	else if(lcmd == CMD_WRITE_CHECKSUM)//write cksum to aprom last
	{
		cktotallen = inpw(pSrc);
		lcksum = inpw(pSrc+4);
		CheckCksumBase();
		ReadData(g_ckbase & 0xFFE00, g_ckbase, (uint32_t*)aprom_buf);
		outpw(aprom_buf+PAGE_SIZE - 8, cktotallen);
		outpw(aprom_buf+PAGE_SIZE - 4, lcksum);
		FMC_Erase(g_ckbase & 0xFFE00);
		WriteData(g_ckbase & 0xFFE00, g_ckbase + 8, (uint32_t*)aprom_buf);
	}
#endif
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCON)&0x2)? 2 : 1);
	}
	else if(lcmd == CMD_SET_APPINFO)
	{
		GetDataFlashInfo(g_apromSize, &g_dataFlashAddr, &g_dataFlashSize);
		
		appInfo = inpw(pSrc);
		if(g_dataFlashSize != 0)
		{			
			FMC_Erase(g_dataFlashAddr);
			FMC_Write(g_dataFlashAddr, appInfo);
			FMC_Write(g_dataFlashAddr+4, TotalLen_bak);
		}
	}
	else if(lcmd == CMD_GET_APPINFO)
	{
		GetDataFlashInfo(g_apromSize, &g_dataFlashAddr, &g_dataFlashSize);
		
		if(g_dataFlashSize != 0)
		{
			FMC_Read(g_dataFlashAddr, &appInfo);
			outpw(response+8, appInfo);
		}
		else
			outpw(response+8, 0xFFFFFFFF);
	}
	
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)//g_dataFlashAddr
				EraseAP(g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			//StartAddress = 0;
			StartAddress = gAppBase;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;
		//printf("StartAddress=%x,TotalPadLen=%d\n",StartAddress, TotalPadLen);
		//return 0;
	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(g_apromSize, &g_dataFlashAddr, &g_dataFlashSize);
    
		goto out;
	}
	else if(lcmd == CMD_RESEND_PACKET)//for APROM&Data flash only
	{  	   
		StartAddress -= LastDataLen;
		TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%PAGE_SIZE) >= (PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+PAGE_SIZE);
		goto out;
    
	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
			TotalLen -= srclen;
		else{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
	    }
    	
		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		//memset(pSrc, 0, srclen);
		{
			uint32_t *p = (uint32_t*)pSrc;
			for(i = 0; i<srclen/4; i++)
				p[i] = 0;
		}
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	//NVIC_DisableIRQ(USBD_IRQn);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;
	//NVIC_EnableIRQ(USBD_IRQn);
	//TotalLen -= srclen;
    
	return 0;
}   
    
//unit is 0.5us
void SysTimerDelay(uint32_t us)
{   
	//SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
	SysTick->LOAD = us * 16; /* using 32MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock
    
    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}   
    
#ifdef SUPPORT_WRITECKSUM
static void CheckCksumBase()
{   
	//skip data flash
	unsigned int aprom_end = g_apromSize, data;
	int result;
	
	result = FMC_Read(0x1F000 - 8, &data);
	if(result == 0)//128K flash
    {
    	FMC_Read(Config0, &data);
    	if((data&0x01)==0)//DFEN enable
    	{
    		FMC_Read(Config1, &aprom_end);
    	}
    	g_ckbase = aprom_end - 8;
    	return;
    }
    else// less than 128K
    {
    	aprom_end = 0x10000;//64K
    	do{
		    result = FMC_Read(aprom_end - 8, &data);
		    if(result == 0)
		    	break;
    		aprom_end = aprom_end/2;
    	}while(aprom_end > 4096);
    
		g_ckbase = aprom_end - 8;
	}
}   
#endif
    
//the smallest of APROM size is 2K 
static __inline uint32_t GetApromSize()
{   
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}   
    
    
    
int32_t main()
{   
    extern uint32_t SystemCoreClock;
    volatile uint32_t pinst;
    uint8_t volatile	bufhead_bak=0;
#ifdef SUPPORT_WRITECKSUM
    uint32_t totallen, cksum;
#endif
	
    UNLOCKREG();
    
	CLK->PWRCTL_BITS.HIRC_EN = 1;
    /* Waiting for 12M/HIRC:12M Xtal stalble */
    while (CLK->CLKSTATUS_BITS.HIRC_STB != 1);
    //SYSCLK->CLKSEL0.HCLK_S = 4;//4:22M
    FMC->ISPCON_BITS.ISPEN = 1;
	
	//FMC_SetVectorPage(0x400);//for test only
	
	g_apromSize = GetApromSize();
	GetDataFlashInfo(g_apromSize, &g_dataFlashAddr, &g_dataFlashSize);
	
	g_pdid = GCR->PDID;        
    
    UartInit();
    //printf("nano\n");
    
#ifndef UART_ONLY
    
	CLK->PLLCTL = 0x20220;//HIRC
    
    /* Waiting for PLL stalble */
	while (CLK->CLKSTATUS_BITS.PLL_STB != 1);
	outpw(&CLK->CLKDIV0, 0x12);//HCLK = PLL/3; USB = PLL/2
	CLK->CLKSEL0 = 2;
    
	/* Process USB event by interrupt */    
	UsbHid();
#endif    


_ISP:	
    while(1)
    {
		if(bUsingUART)
		{
    		if(bUartDataReady == TRUE)
	   		{
    			bUartDataReady = FALSE;
	    		g_timecnt = 0;
    			ParseCmd(uart_rcvbuf, HID_MAX_PACKET_SIZE_EP1, FALSE);
	   			PutString();
	    	}
    		//timeout happen; but byte is less than 64 bytes; host goes wrong
			if(bufhead > 0)
			{
				if(g_timecnt == 0)
					bufhead_bak = bufhead;
				SysTimerDelay(1);//0.5us
				g_timecnt++;
				if(g_timecnt > 2000)//1ms
				{					
					g_timecnt = 0;
					if(bufhead_bak == bufhead)
					{
						bufhead = 0;
					}
				}
			}
		}
#ifndef UART_ONLY		
		else
		{
    	
	   		if(bUsbDataReady == TRUE)
	    	{
    			ParseCmd(g_HID_sDevice.au8IntOutBuffer, HID_MAX_PACKET_SIZE_EP1, TRUE);
    			bUsbDataReady = FALSE;
				_DRVUSB_TRIG_EP(3,HID_MAX_PACKET_SIZE_EP1);
	    	}
 		
			if((g_pdid & 0x000FFF00) == 0x00010100)
			{    	
    			pinst = (GPIOD->PIN & (1 << 0));//GPD0 low
		    }
    		else
    		{
				pinst = (GPIOB->PIN & (1 << 15));//GPB15 low
			}
    		if(pinst != 0)//GPIO high
    			goto _APROM;
		}
#endif
	}
}   
    
#ifndef UART_ONLY
void my_memcpy(void *dest, void *src, int32_t size)
{   
#if 0//if ATTR[HSIZE_8] = 0
    //if((uint32_t)dest &0x3)
    //    while(1);
    //if((uint32_t)src &0x3)
    //    while(1);
    
    size = size + 0x3 & 0xFFFFFFFCUL;
    
    memcpy(dest, src, size);
#else//if ATTR[HSIZE_8] = 1
	WordsCpy(dest, src, size);
#endif
}   
#endif
    

