/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "NANO1xx.h" 

#define FISPCON         (FMC_BASE+0x000)			
#define ISPADR         (FMC_BASE+0x004)		
#define ISPDAT         (FMC_BASE+0x008)			
#define ISPCMD         (FMC_BASE+0x00C)			
#define ISPTRG         (FMC_BASE+0x010)		

#define ISPGO           0x01
#define ISPFF           0x00000040

#define ISP_Read        0x00
#define ISP_VecRemap	0x2E			/*!<ISP Command - Vector Page Re-Map */

#define Config0         0x00300000
#define Config1         0x00300004 

typedef void (FUNC_PTR)(void);

__asm spChange(uint32_t _sp)
{
	MSR MSP, r0
	BX lr
}
    

int32_t FMC_SetVectorPage(uint32_t u32addr)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_VecRemap);
    outp32(ISPADR, u32addr);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }

	return 0;
}

int FMC_Read(unsigned int address, unsigned int * data)
{
    unsigned int Reg;

    outp32(ISPCMD, ISP_Read);
    outp32(ISPADR, address);
    outp32(ISPDAT, 0x00000000);
	outp32(ISPTRG, ISPGO); 
    
    __ISB();

    Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
 
	*data = inp32(ISPDAT);

    return 0;
}

int32_t main()
{     	
    volatile uint32_t a, b, _sp;
	FUNC_PTR  		*func;
	uint32_t uData;
	
	UNLOCKREG();
	FMC->ISPCON_BITS.ISPEN = 1;
		
	FMC_Read(Config0, &uData);
	
	if(uData & 0x1)
		b = 0x400;
	else
	{
		FMC_Read(Config1, &uData);
		FMC_Read(uData, &a);

		if((a == 0xFFFFFFFF) || (a < 0x400))
			b = 0x400;
		else 
			b = a;
	}
	
	FMC_SetVectorPage(b);
	_sp = *(volatile uint32_t *) b;	

	spChange(_sp);		
	
	func = (FUNC_PTR *)(*(uint32_t *)(b + 4));
	func();
		

}   
    


