;/******************************************************************************
; * @file:   startup_nano1xx_vecmap_1M.s
; * @brief   CMSIS ARM Cortex-M0 Core Device Startup File
; * @version 1.0.0
; * @date    18,October, 2012
; *
; * Copyright (C) 2012 Nuvoton Technology Corp. All rights reserved.
; *****************************************************************************/


; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                DCD     BOD_IRQHandler            ; Brownout low voltage detected interrupt  
                DCD     WDT_IRQHandler            ; Watch Dog Timer interrupt  
                DCD     EINT0_IRQHandler          ; External signal interrupt from PB.14 pin
                DCD     EINT1_IRQHandler          ; External signal interrupt from PB.15 pin
                DCD     GPAB_IRQHandler           ; External signal interrupt from PA[15:0] / PB[13:0]  
                DCD     GPCDE_IRQHandler          ; External interrupt from PC[15:0]/PD[15:0]/PE[15:0]
                DCD     PWM0_IRQHandler           ; PWM 0 interrupt 
                DCD     PWM1_IRQHandler           ; PWM 1 interrupt  
                DCD     TMR0_IRQHandler           ; Timer 0 interrupt
                DCD     TMR1_IRQHandler           ; Timer 1 interrupt  
                DCD     TMR2_IRQHandler           ; Timer 2 interrupt 
                DCD     TMR3_IRQHandler           ; Timer 3 interrupt 
                DCD     UART0_IRQHandler          ; UART0 interrupt
                DCD     UART1_IRQHandler          ; UART1 interrupt
                DCD     SPI0_IRQHandler           ; SPI0 interrupt 
                DCD     SPI1_IRQHandler           ; SPI1 interrupt 
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved 
                DCD     I2C0_IRQHandler           ; I2C0 interrupt 
                DCD     I2C1_IRQHandler           ; I2C1 interrupt 
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     USBD_IRQHandler           ; USB FS Device interrupt 
                DCD     TK_IRQHandler             ; Touch key interrupt  
                DCD     LCD_IRQHandler            ; LCD interrupt 
                DCD     PDMA_IRQHandler           ; PDMA interrupt
                DCD     I2S_IRQHandler            ; I2S interrupt 
                DCD     PDWU_IRQHandler           ; Power Down Wake up interrupt
                DCD     ADC_IRQHandler            ; ADC interrupt
                DCD     DAC_IRQHandler            ; DAC interrupt
                DCD     RTC_IRQHandler            ; Real time clock interrupt
                AREA    |.text|, CODE, READONLY


; Reset Handler

Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                ;IMPORT  SystemInit
                IMPORT  __main

; UNLOCKREG()
                LDR     r0,UNLOCKREG           ;LDR     r0,[pc,#66]    
                MOVS    r1,#0x59
                STR     r1,[r0,#0]
                MOVS    r1,#0x16
                STR     r1,[r0,#0]
                MOVS    r1,#0x88
                STR     r1,[r0,#0]

;FMC->ISPCON_BITS.ISPEN = 1             
                LDR     r0,ISPCON               ;LDR     r0,[pc,#58]     
                LDR     r1,[r0,#0]
                MOVS    r2,#1
                ORRS    r1,r1,r2
                STR     r1,[r0,#0]
;FMC->ISPCMD_BITS.FCTRL = 0xE;
                LDR     r1,[r0,#0xc]
                LSRS    r1,r1,#4
                LSLS    r1,r1,#4
                ADDS    r1,r1,#0xe
                STR     r1,[r0,#0xc]
;FMC->ISPCMD_BITS.FCEN = 0;                
                LDR     r1,[r0,#0xc]
                MOVS    r3,#0x10
                BICS    r1,r1,r3
                STR     r1,[r0,#0xc]
;FMC->ISPCMD_BITS.FOEN = 1;                
                LDR     r1,[r0,#0xc]
                MOVS    r3,#0x20
                ORRS    r1,r1,r3
                STR     r1,[r0,#0xc]
;Read 0x1EA00 
				LDR		r3, DATAFLASH
				LDR		r1,[r3, #0]
				CMP		r1, #2 
;FMC->ISPADR	= 0xC800; or 512 
				BEQ		%F2               
                LDR    r3, APP1BASE
1				
                STR     r3,[r0,#4]
;FMC->ISPTRG_BITS.ISPGO = 1;	                
                LDR     r1,[r0,#0x10]
                ORRS    r1,r1,r2
                STR     r1,[r0,#0x10]
;change SP
				LDR r0, [r3, #0]
				MSR MSP, r0
				LDR r0, [r3, #4]
				BX r0					
2              
				LDR    r3, APP2BASE
				B 		%B1  
                ;LDR     R0, =__main
                ;BX      R0

                DCW     0
UNLOCKREG       DCD     1342177536         ; 0x50000100
ISPCON          DCD     1342226432         ; 0x5000C000
DATAFLASH		DCD		125440			   ; 0x1EA00
APP1BASE		DCD		512			       ; 0x200
APP2BASE		DCD		51200			   ; 0xC800

;ISPCON          DCD     1342226436         ; 0x5000C004
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)                

NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  BOD_IRQHandler            [WEAK]
                EXPORT  WDT_IRQHandler            [WEAK]
                EXPORT  EINT0_IRQHandler          [WEAK]
                EXPORT  EINT1_IRQHandler          [WEAK]
                EXPORT  GPAB_IRQHandler           [WEAK]
                EXPORT  GPCDE_IRQHandler          [WEAK]
                EXPORT  PWM0_IRQHandler           [WEAK]
                EXPORT  PWM1_IRQHandler           [WEAK]
                EXPORT  TMR0_IRQHandler           [WEAK]
                EXPORT  TMR1_IRQHandler           [WEAK]
                EXPORT  TMR2_IRQHandler           [WEAK]
                EXPORT  TMR3_IRQHandler           [WEAK]
                EXPORT  UART0_IRQHandler          [WEAK]
                EXPORT  UART1_IRQHandler          [WEAK]
                EXPORT  SPI0_IRQHandler           [WEAK]
                EXPORT  SPI1_IRQHandler           [WEAK]
                EXPORT  I2C0_IRQHandler           [WEAK]
                EXPORT  I2C1_IRQHandler           [WEAK]
                EXPORT  USBD_IRQHandler           [WEAK]
                EXPORT  TK_IRQHandler             [WEAK]
                EXPORT  LCD_IRQHandler            [WEAK]
                EXPORT  PDMA_IRQHandler           [WEAK]
				EXPORT  I2S_IRQHandler            [WEAK]
                EXPORT  PDWU_IRQHandler           [WEAK]
                EXPORT  ADC_IRQHandler            [WEAK]
                EXPORT  DAC_IRQHandler            [WEAK]
                EXPORT  RTC_IRQHandler            [WEAK]
                
BOD_IRQHandler
WDT_IRQHandler
EINT0_IRQHandler
EINT1_IRQHandler
GPAB_IRQHandler
GPCDE_IRQHandler
PWM0_IRQHandler
PWM1_IRQHandler
TMR0_IRQHandler
TMR1_IRQHandler
TMR2_IRQHandler
TMR3_IRQHandler
UART0_IRQHandler
UART1_IRQHandler
SPI0_IRQHandler
SPI1_IRQHandler
I2C0_IRQHandler
I2C1_IRQHandler
USBD_IRQHandler
TK_IRQHandler
LCD_IRQHandler
PDMA_IRQHandler
I2S_IRQHandler
PDWU_IRQHandler
ADC_IRQHandler
DAC_IRQHandler
RTC_IRQHandler

                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
