#include <stdio.h>
#include "NANO1xx.h"
//#include "Platform.h"
#include "Driver/DrvUSB.h"
#include "Driver/DrvGPIO.h"
#include "USB/HIDDevice.h"
#include "USB/USBInfra.h"
#include "FMC.h"

#pragma arm section code = "FMC_Write"
int FMC_Write(unsigned int address, unsigned int data)
{
    unsigned int Reg;
    
    outp32(ISPCMD, ISP_Program);
    outp32(ISPADR, address);
    outp32(ISPDAT, data);
    outp32(ISPTRG, ISPGO); 
    
    __ISB();
    
    Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    return 0;
}

#pragma arm section code = "FMC_Read"
int FMC_Read(unsigned int address, unsigned int * data)
{
    unsigned int Reg;

    outp32(ISPCMD, ISP_Read);
    outp32(ISPADR, address);
    outp32(ISPDAT, 0x00000000);
	outp32(ISPTRG, ISPGO); 
    
    __ISB();

    Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
 
	*data = inp32(ISPDAT);

    return 0;
}

#pragma arm section code = "FMC_Erase"
int FMC_Erase(unsigned int address)
{
    unsigned int Reg;
    
    outp32(ISPCMD, ISP_PageErase);
    outp32(ISPADR, address);
    outp32(ISPTRG, ISPGO); 

	__ISB();
    
    Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    return 0;
}

#pragma arm section code = "FMC_SetVectorPage"
int32_t FMC_SetVectorPage(uint32_t u32addr)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_VecRemap);
    outp32(ISPADR, u32addr);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }

	return 0;
}

#pragma arm section code = "FMC_ReadCID"
int32_t FMC_ReadCID(uint32_t * data)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_ReadCID);
    outp32(ISPADR, 0);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    *data = inp32(ISPDAT);

	return 0;
}


/**
  * @brief  Read device ID.
  * @param  u32data: A word buffer to store the device ID. 
  * @retval E_FMC_OK: Success
  *         E_FMC_ISP_FAIL: ISP command failed.
  */
#pragma arm section code = "FMC_ReadDID"
int32_t FMC_ReadDID(uint32_t * data)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_ReadDID);
    outp32(ISPADR, 0);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    
    *data = inp32(ISPDAT);

	return 0;
}


/**
  * @brief  Read product ID.
  * @param  u32data: A word buffer to store the product ID. 
  * @retval E_FMC_OK: Success
  *         E_FMC_ISP_FAIL: ISP command failed.
  */
#pragma arm section code = "FMC_ReadPID"
int32_t FMC_ReadPID(uint32_t *data)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_ReadDID);
    outp32(ISPADR, 0);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    
    *data = inp32(ISPDAT);

	return 0;
}


/**
  * @brief  Read unique ID. There's three unique ID.
  * @param  index: Unique ID index. Index should be 0, 1, or 2.
  *         u32data: A word buffer to store the unique ID. 
  * @retval E_FMC_OK: Success
  *         E_FMC_ISP_FAIL: ISP command failed.
  *         E_FMC_INVALID_PARAM: Invalid unique ID index.
  */
#pragma arm section code = "FMC_ReadUID"
int32_t FMC_ReadUID(int index, uint32_t *data)
{
	unsigned int Reg;
	
	outp32(ISPCMD, ISP_ReadUID);
    outp32(ISPADR, 0);
    outp32(ISPTRG, ISPGO); 
	
	__ISB();

	Reg = inp32(FISPCON);
    if (Reg & ISPFF)
    {
    	outp32(FISPCON, Reg);
    	return -1;
    }
    
    *data = inp32(ISPDAT);

	return 0;
}


#pragma arm section code = "ReadData"
void ReadData(unsigned int addr_start, unsigned int addr_end, unsigned int* data)    // Read data from flash
{
    unsigned int rLoop;

    for ( rLoop = addr_start; rLoop < addr_end; rLoop += 4 ) 
    {     
		FMC_Read(rLoop, data);
		data++;
    }
    return;
}

#pragma arm section code = "WriteData"
void WriteData(unsigned int addr_start, unsigned int addr_end, unsigned int *data)  // Write data into flash
{
    unsigned int wLoop;
    
    for ( wLoop = addr_start; wLoop < addr_end; wLoop+=4 ) 
    {
        FMC_Write(wLoop, *data);
        data++;
    }
}



#pragma arm section code = "EraseAP"
//bAprom == TRUE erase all aprom besides data flash
void EraseAP(unsigned int addr_start, unsigned int addr_end)
{
    unsigned int eraseLoop;
    unsigned int erase_end;
    
    //if(bAprom == TRUE)
    //{
    	
    	//erase_end = g_dataFlashAddr;
    	//eraseLoop = APROM_BASE;
    //}
    //else
    {
    	eraseLoop = addr_start;
    	erase_end = addr_end;
    }
    for ( ; eraseLoop < erase_end; eraseLoop += PAGE_SIZE )
    {
        FMC_Erase(eraseLoop);
    }           
    return;
}


#pragma arm section code = "UpdateConfig"
void UpdateConfig(unsigned int *data, unsigned int *res)
{
	
	FMC->ISPCON_BITS.CFGUEN = 1;//enable config update
    FMC_Erase(Config0);
    //FMC_Write(Config0, 0xFFFFFF7F); // Disable Lock bit
    //FMC_Write(Config1, 0x0001F000);
    
   	FMC_Write(Config0, *data);
   	FMC_Write(Config1, *(data+1));
    
    if(res)
    {
	    FMC_Read(Config0, res);
    	FMC_Read(Config1, res+1);
    }
    
    FMC->ISPCON_BITS.CFGUEN = 0;//disable config update

}

#pragma arm section code = "GetDataFlashInfo"
void GetDataFlashInfo(uint32_t apromsize, uint32_t *addr, uint32_t *size)
{
	uint32_t uData;
	
	*size = 0;
	
    {
    	FMC_Read(Config0, &uData);
    	if((uData&0x01)==0)//DFEN enable
    	{
    		FMC_Read(Config1, &uData);
    		if(uData > apromsize || uData < 0x200)//avoid config1 value from error
    			uData = apromsize;
    		
    		*addr = uData;
    		*size = apromsize - uData;
    	}
    	else
    	{
    		*addr = apromsize;
    		*size = 0;
    	}
    }
}
#pragma arm section code

int main()
{
	return 0;
}
